@@ -4,5 +4,5 @@ fn main() {
     let ptr = Box::into_raw(Box::new(0u32));
     // Can't start with an integer pointer and get to something usable
     let ptr = (1 as *mut u8).wrapping_offset(ptr as isize);
-    let _ = unsafe { *ptr };
+    let _val = unsafe { *ptr };
 }
