@@ -3,7 +3,7 @@ fn foo(x: &mut (i32, i32)) -> Option<&i32> {
     let xraw = x as *mut (i32, i32);
     let ret = Some(unsafe { &(*xraw).1 });
     unsafe { *xraw = (42, 23) }; // unfreeze
-    ret //~ ERROR is not frozen
+    ret //~ ERROR borrow stack
 }
 
 fn main() {
