@@ -7,7 +7,7 @@
 
 #![feature(rustc_private, drain_filter)]
 #![warn(clippy::all, rust_2018_idioms)]
-#![allow(clippy::cyclomatic_complexity, clippy::too_many_arguments, clippy::redundant_closure)]
+#![allow(clippy::cognitive_complexity, clippy::too_many_arguments, clippy::redundant_closure)]
 
 #[macro_use]
 extern crate failure;
