@@ -3,7 +3,7 @@ use std::ops::Deref;
 
 use syntax::ast::{self, FunctionRetTy, Mutability};
 use syntax::source_map::{self, BytePos, Span};
-use syntax::symbol::keywords;
+use syntax::symbol::kw;
 
 use crate::config::lists::*;
 use crate::config::{IndentStyle, TypeDensity};
@@ -24,14 +24,14 @@ use crate::utils::{
 };
 
 #[derive(Copy, Clone, Debug, Eq, PartialEq)]
-pub enum PathContext {
+pub(crate) enum PathContext {
     Expr,
     Type,
     Import,
 }
 
 // Does not wrap on simple segments.
-pub fn rewrite_path(
+pub(crate) fn rewrite_path(
     context: &RewriteContext<'_>,
     path_context: PathContext,
     qself: Option<&ast::QSelf>,
@@ -106,7 +106,7 @@ where
 
     for segment in iter {
         // Indicates a global path, shouldn't be rendered.
-        if segment.ident.name == keywords::PathRoot.name() {
+        if segment.ident.name == kw::PathRoot {
             continue;
         }
         if first {
@@ -133,11 +133,11 @@ where
 }
 
 #[derive(Debug)]
-pub enum SegmentParam<'a> {
+pub(crate) enum SegmentParam<'a> {
     Const(&'a ast::AnonConst),
     LifeTime(&'a ast::Lifetime),
     Type(&'a ast::Ty),
-    Binding(&'a ast::TypeBinding),
+    Binding(&'a ast::AssocTyConstraint),
 }
 
 impl<'a> SegmentParam<'a> {
@@ -167,16 +167,18 @@ impl<'a> Rewrite for SegmentParam<'a> {
             SegmentParam::Const(const_) => const_.rewrite(context, shape),
             SegmentParam::LifeTime(lt) => lt.rewrite(context, shape),
             SegmentParam::Type(ty) => ty.rewrite(context, shape),
-            SegmentParam::Binding(binding) => {
+            SegmentParam::Binding(assoc_ty_constraint) => {
                 let mut result = match context.config.type_punctuation_density() {
-                    TypeDensity::Wide => format!("{} = ", rewrite_ident(context, binding.ident)),
+                    TypeDensity::Wide => {
+                        format!("{} = ", rewrite_ident(context, assoc_ty_constraint.ident))
+                    }
                     TypeDensity::Compressed => {
-                        format!("{}=", rewrite_ident(context, binding.ident))
+                        format!("{}=", rewrite_ident(context, assoc_ty_constraint.ident))
                     }
                 };
                 let budget = shape.width.checked_sub(result.len())?;
-                let rewrite = binding
-                    .ty
+                let rewrite = assoc_ty_constraint
+                    .kind
                     .rewrite(context, Shape::legacy(budget, shape.indent + result.len()))?;
                 result.push_str(&rewrite);
                 Some(result)
@@ -185,6 +187,15 @@ impl<'a> Rewrite for SegmentParam<'a> {
     }
 }
 
+impl Rewrite for ast::AssocTyConstraintKind {
+    fn rewrite(&self, context: &RewriteContext<'_>, shape: Shape) -> Option<String> {
+        match self {
+            ast::AssocTyConstraintKind::Equality { ty } => ty.rewrite(context, shape),
+            ast::AssocTyConstraintKind::Bound { bounds } => bounds.rewrite(context, shape),
+        }
+    }
+}
+
 // Formats a path segment. There are some hacks involved to correctly determine
 // the segment's associated span since it's not part of the AST.
 //
@@ -216,13 +227,13 @@ fn rewrite_segment(
     if let Some(ref args) = segment.args {
         match **args {
             ast::GenericArgs::AngleBracketed(ref data)
-                if !data.args.is_empty() || !data.bindings.is_empty() =>
+                if !data.args.is_empty() || !data.constraints.is_empty() =>
             {
                 let param_list = data
                     .args
                     .iter()
                     .map(SegmentParam::from_generic_arg)
-                    .chain(data.bindings.iter().map(|x| SegmentParam::Binding(&*x)))
+                    .chain(data.constraints.iter().map(|x| SegmentParam::Binding(&*x)))
                     .collect::<Vec<_>>();
 
                 // HACK: squeeze out the span between the identifier and the parameters.
@@ -320,7 +331,7 @@ where
     let list_lo = context.snippet_provider.span_after(span, "(");
     let (list_str, tactic) = if inputs.len() == 0 {
         let tactic = get_tactics(&[], &output, shape);
-        let list_hi = context.snippet_provider.span_before_last(span, ")");
+        let list_hi = context.snippet_provider.span_before(span, ")");
         let comment = context
             .snippet_provider
             .span_to_snippet(mk_sp(list_lo, list_hi))?
@@ -827,7 +838,11 @@ fn join_bounds(
     Some(result)
 }
 
-pub fn can_be_overflowed_type(context: &RewriteContext<'_>, ty: &ast::Ty, len: usize) -> bool {
+pub(crate) fn can_be_overflowed_type(
+    context: &RewriteContext<'_>,
+    ty: &ast::Ty,
+    len: usize,
+) -> bool {
     match ty.node {
         ast::TyKind::Tup(..) => context.use_block_indent() && len == 1,
         ast::TyKind::Rptr(_, ref mutty) | ast::TyKind::Ptr(ref mutty) => {
